{
  This file was automatically generated on 2012-10-01.

  If you have a bugfix for this file and want to commit it,
  please fix the bug in the generator. You can find a link
  to the generator git on tinkerforge.com
}

unit BrickletAnalogOut;

{$ifdef FPC}{$mode OBJFPC}{$H+}{$endif}

interface

uses
  Device, IPConnection, LEConverter;

const
  BRICKLET_ANALOG_OUT_FUNCTION_SET_VOLTAGE = 1;
  BRICKLET_ANALOG_OUT_FUNCTION_GET_VOLTAGE = 2;
  BRICKLET_ANALOG_OUT_FUNCTION_SET_MODE = 3;
  BRICKLET_ANALOG_OUT_FUNCTION_GET_MODE = 4;

type
  /// <summary>
  ///  Device for output of voltage between 0 and 5V
  /// </summary>
  TBrickletAnalogOut = class(TDevice)
  private
  protected
  public
    /// <summary>
    ///  Creates an object with the unique device ID <c>uid</c>. This object can
    ///  then be added to the IP connection.
    /// </summary>
    constructor Create(const uid_: string);

    /// <summary>
    ///  Sets the voltage in mV. The possible range is 0V to 5V (0-5000).
    ///  Calling this function will set the mode to 0 (see `:func:SetMode`).
    ///  
    ///  The default value is 0 (with mode 1).
    /// </summary>
    procedure SetVoltage(const voltage: word); virtual;

    /// <summary>
    ///  Returns the voltage as set by <see cref="BrickletAnalogOut.TBrickletAnalogOut.SetVoltage"/>.
    /// </summary>
    function GetVoltage: word; virtual;

    /// <summary>
    ///  Sets the mode of the analog value. Possible modes:
    ///  
    ///  * 0: Normal Mode (Analog value as set by <see cref="BrickletAnalogOut.TBrickletAnalogOut.SetVoltage"/> is applied
    ///  * 1: 1k Ohm resistor to ground
    ///  * 2: 100k Ohm resistor to ground
    ///  * 3: 500k Ohm resistor to ground
    ///  
    ///  Setting the mode to 0 will result in an output voltage of 0. You can jump
    ///  to a higher output voltage directly by calling <see cref="BrickletAnalogOut.TBrickletAnalogOut.SetVoltage"/>.
    ///  
    ///  The default mode is 1.
    /// </summary>
    procedure SetMode(const mode: byte); virtual;

    /// <summary>
    ///  Returns the mode as set by <see cref="BrickletAnalogOut.TBrickletAnalogOut.SetMode"/>.
    /// </summary>
    function GetMode: byte; virtual;
  end;

implementation

constructor TBrickletAnalogOut.Create(const uid_: string);
begin
  inherited Create(uid_);
  expectedName := 'Analog Out Bricklet';
  bindingVersion[0] := 1;
  bindingVersion[1] := 0;
  bindingVersion[2] := 0;
end;

procedure TBrickletAnalogOut.SetVoltage(const voltage: word);
var request: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_ANALOG_OUT_FUNCTION_SET_VOLTAGE, 6);
  LEConvertUInt16To(voltage, 4, request);
  SendRequestNoResponse(request);
end;

function TBrickletAnalogOut.GetVoltage: word;
var request, response: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_ANALOG_OUT_FUNCTION_GET_VOLTAGE, 4);
  SetLength(response, 0);
  SendRequestExpectResponse(request, BRICKLET_ANALOG_OUT_FUNCTION_GET_VOLTAGE, response);
  result := LEConvertUInt16From(4, response);
end;

procedure TBrickletAnalogOut.SetMode(const mode: byte);
var request: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_ANALOG_OUT_FUNCTION_SET_MODE, 5);
  LEConvertUInt8To(mode, 4, request);
  SendRequestNoResponse(request);
end;

function TBrickletAnalogOut.GetMode: byte;
var request, response: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_ANALOG_OUT_FUNCTION_GET_MODE, 4);
  SetLength(response, 0);
  SendRequestExpectResponse(request, BRICKLET_ANALOG_OUT_FUNCTION_GET_MODE, response);
  result := LEConvertUInt8From(4, response);
end;

end.
