{
  This file was automatically generated on 2012-10-01.

  If you have a bugfix for this file and want to commit it,
  please fix the bug in the generator. You can find a link
  to the generator git on tinkerforge.com
}

unit BrickletIndustrialQuadRelay;

{$ifdef FPC}{$mode OBJFPC}{$H+}{$endif}

interface

uses
  Device, IPConnection, LEConverter;

const
  BRICKLET_INDUSTRIAL_QUAD_RELAY_FUNCTION_SET_VALUE = 1;
  BRICKLET_INDUSTRIAL_QUAD_RELAY_FUNCTION_GET_VALUE = 2;
  BRICKLET_INDUSTRIAL_QUAD_RELAY_FUNCTION_SET_MONOFLOP = 3;
  BRICKLET_INDUSTRIAL_QUAD_RELAY_FUNCTION_GET_MONOFLOP = 4;
  BRICKLET_INDUSTRIAL_QUAD_RELAY_FUNCTION_SET_GROUP = 5;
  BRICKLET_INDUSTRIAL_QUAD_RELAY_FUNCTION_GET_GROUP = 6;
  BRICKLET_INDUSTRIAL_QUAD_RELAY_FUNCTION_GET_AVAILABLE_FOR_GROUP = 7;
  BRICKLET_INDUSTRIAL_QUAD_RELAY_CALLBACK_MONOFLOP_DONE = 8;

type
  TArray0To3OfChar = array [0..3] of char;

  TBrickletIndustrialQuadRelayNotifyMonoflopDone = procedure(const pinMask: word; const valueMask: word) of object;

  /// <summary>
  ///  Device for controlling up to 4 Solid State Relays
  /// </summary>
  TBrickletIndustrialQuadRelay = class(TDevice)
  private
    monoflopDoneCallback: TBrickletIndustrialQuadRelayNotifyMonoflopDone;
  protected
    procedure CallbackWrapperMonoflopDone(const packet: TByteArray); virtual;
  public
    /// <summary>
    ///  Creates an object with the unique device ID <c>uid</c>. This object can
    ///  then be added to the IP connection.
    /// </summary>
    constructor Create(const uid_: string);

    /// <summary>
    ///  Sets the output value with a bitmask. The bitmask
    ///  is 16 bit long, *true* refers to a closed relay and *false* refers to 
    ///  an open relay.
    ///  
    ///  For example: The value 0b0000000000000011 will close the relay 
    ///  of pins 0-1 and open the other pins.
    ///  
    ///  If no groups are used (see <see cref="BrickletIndustrialQuadRelay.TBrickletIndustrialQuadRelay.SetGroup"/>), the pins correspond to the
    ///  markings on the Quad Relay Bricklet.
    ///  
    ///  If groups are used, the pins correspond to the element in the group.
    ///  Element 1 in the group will get pins 0-3, element 2 pins 4-7, element 3
    ///  pins 8-11 and element 4 pins 12-15.
    /// </summary>
    procedure SetValue(const valueMask: word); virtual;

    /// <summary>
    ///  Returns the bitmask as set by <see cref="BrickletIndustrialQuadRelay.TBrickletIndustrialQuadRelay.SetValue"/>.
    /// </summary>
    function GetValue: word; virtual;

    /// <summary>
    ///  Configures a monoflop of the pins specified by the first parameter
    ///  bitmask.
    ///  
    ///  The second parameter is a bitmask with the desired value of the specified
    ///  pins (*true* means relay closed and *false* means relay open).
    ///  
    ///  The third parameter indicates the time (in ms) that the pins should hold
    ///  the value.
    ///  
    ///  If this function is called with the parameters 
    ///  ((1 &lt;&lt; 0) | (1 &lt;&lt; 3), (1 &lt;&lt; 0), 1500):
    ///  Pin 0 will close and pin 3 will open. In 1.5s pin 0 will open and pin
    ///  3 will close again.
    ///  
    ///  A monoflop can be used as a fail-safe mechanism. For example: Lets assume you
    ///  have a RS485 bus and a Quad Relay Bricklet connected to one of the slave
    ///  stacks. You can now call this function every second, with a time parameter
    ///  of two seconds and pin 0 closed. Pin 0 will be closed all the time. If now
    ///  the RS485 connection is lost, then pin 0 will be opened in at most two seconds.
    /// </summary>
    procedure SetMonoflop(const pinMask: word; const valueMask: word; const time: longword); virtual;

    /// <summary>
    ///  Returns (for the given pin) the current value and the time as set by
    ///  <see cref="BrickletIndustrialQuadRelay.TBrickletIndustrialQuadRelay.SetMonoflop"/> as well as the remaining time until the value flips.
    ///  
    ///  If the timer is not running currently, the remaining time will be returned
    ///  as 0.
    /// </summary>
    procedure GetMonoflop(const pin: byte; out value: word; out time: longword; out timeRemaining: longword); virtual;

    /// <summary>
    ///  Sets a group of Quad Relay Bricklets that should work together. You can
    ///  find Bricklets that can be grouped together with <see cref="BrickletIndustrialQuadRelay.TBrickletIndustrialQuadRelay.GetAvailableForGroup"/>.
    ///  
    ///  The group consists of 4 elements. Element 1 in the group will get pins 0-3,
    ///  element 2 pins 4-7, element 3 pins 8-11 and element 4 pins 12-15.
    ///  
    ///  Each element can either be one of the ports ('a' to 'd') or 'n' if it should
    ///  not be used.
    ///  
    ///  For example: If you have two Quad Relay Bricklets connected to port A and
    ///  port B respectively, you could call with "['a', 'b', 'n', 'n']".
    ///  
    ///  Now the pins on the Quad Relay on port A are assigned to 0-3 and the
    ///  pins on the Quad Relay on port B are assigned to 4-7. It is now possible
    ///  to call <see cref="BrickletIndustrialQuadRelay.TBrickletIndustrialQuadRelay.SetValue"/> and control two Bricklets at the same time.
    /// </summary>
    procedure SetGroup(const group: TArray0To3OfChar); virtual;

    /// <summary>
    ///  Returns the group as set by <see cref="BrickletIndustrialQuadRelay.TBrickletIndustrialQuadRelay.SetGroup"/>
    /// </summary>
    function GetGroup: TArray0To3OfChar; virtual;

    /// <summary>
    ///  Returns a bitmask of ports that are available for grouping. For example the
    ///  value 0b0101 means: Port *A* and Port *C* are connected to Bricklets that
    ///  can be grouped together.
    /// </summary>
    function GetAvailableForGroup: byte; virtual;

    /// <summary>
    ///  This callback is triggered whenever a monoflop timer reaches 0. The
    ///  parameters contain the involved pins and the current value of the pins
    ///  (the value after the monoflop).
    /// </summary>
    property OnMonoflopDone: TBrickletIndustrialQuadRelayNotifyMonoflopDone read monoflopDoneCallback write monoflopDoneCallback;
  end;

implementation

constructor TBrickletIndustrialQuadRelay.Create(const uid_: string);
begin
  inherited Create(uid_);
  expectedName := 'Industrial Quad Relay Bricklet';
  bindingVersion[0] := 1;
  bindingVersion[1] := 0;
  bindingVersion[2] := 0;
  callbackWrappers[BRICKLET_INDUSTRIAL_QUAD_RELAY_CALLBACK_MONOFLOP_DONE] := {$ifdef FPC}@{$endif}CallbackWrapperMonoflopDone;
end;

procedure TBrickletIndustrialQuadRelay.SetValue(const valueMask: word);
var request: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_INDUSTRIAL_QUAD_RELAY_FUNCTION_SET_VALUE, 6);
  LEConvertUInt16To(valueMask, 4, request);
  SendRequestNoResponse(request);
end;

function TBrickletIndustrialQuadRelay.GetValue: word;
var request, response: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_INDUSTRIAL_QUAD_RELAY_FUNCTION_GET_VALUE, 4);
  SetLength(response, 0);
  SendRequestExpectResponse(request, BRICKLET_INDUSTRIAL_QUAD_RELAY_FUNCTION_GET_VALUE, response);
  result := LEConvertUInt16From(4, response);
end;

procedure TBrickletIndustrialQuadRelay.SetMonoflop(const pinMask: word; const valueMask: word; const time: longword);
var request: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_INDUSTRIAL_QUAD_RELAY_FUNCTION_SET_MONOFLOP, 12);
  LEConvertUInt16To(pinMask, 4, request);
  LEConvertUInt16To(valueMask, 6, request);
  LEConvertUInt32To(time, 8, request);
  SendRequestNoResponse(request);
end;

procedure TBrickletIndustrialQuadRelay.GetMonoflop(const pin: byte; out value: word; out time: longword; out timeRemaining: longword);
var request, response: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_INDUSTRIAL_QUAD_RELAY_FUNCTION_GET_MONOFLOP, 5);
  LEConvertUInt8To(pin, 4, request);
  SetLength(response, 0);
  SendRequestExpectResponse(request, BRICKLET_INDUSTRIAL_QUAD_RELAY_FUNCTION_GET_MONOFLOP, response);
  value := LEConvertUInt16From(4, response);
  time := LEConvertUInt32From(6, response);
  timeRemaining := LEConvertUInt32From(10, response);
end;

procedure TBrickletIndustrialQuadRelay.SetGroup(const group: TArray0To3OfChar);
var request: TByteArray; i: longint;
begin
  request := CreateRequestPacket(stackID, BRICKLET_INDUSTRIAL_QUAD_RELAY_FUNCTION_SET_GROUP, 8);
  for i := 0 to Length(group) - 1 do LEConvertCharTo(group[i], 4 + (i * 1), request);
  SendRequestNoResponse(request);
end;

function TBrickletIndustrialQuadRelay.GetGroup: TArray0To3OfChar;
var request, response: TByteArray; i: longint;
begin
  request := CreateRequestPacket(stackID, BRICKLET_INDUSTRIAL_QUAD_RELAY_FUNCTION_GET_GROUP, 4);
  SetLength(response, 0);
  SendRequestExpectResponse(request, BRICKLET_INDUSTRIAL_QUAD_RELAY_FUNCTION_GET_GROUP, response);
  for i := 0 to 3 do result[i] := LEConvertCharFrom(4 + (i * 1), response);
end;

function TBrickletIndustrialQuadRelay.GetAvailableForGroup: byte;
var request, response: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_INDUSTRIAL_QUAD_RELAY_FUNCTION_GET_AVAILABLE_FOR_GROUP, 4);
  SetLength(response, 0);
  SendRequestExpectResponse(request, BRICKLET_INDUSTRIAL_QUAD_RELAY_FUNCTION_GET_AVAILABLE_FOR_GROUP, response);
  result := LEConvertUInt8From(4, response);
end;

procedure TBrickletIndustrialQuadRelay.CallbackWrapperMonoflopDone(const packet: TByteArray);
var pinMask: word; valueMask: word;
begin
  if (Assigned(monoflopDoneCallback)) then begin
    pinMask := LEConvertUInt16From(4, packet);
    valueMask := LEConvertUInt16From(6, packet);
    monoflopDoneCallback(pinMask, valueMask);
  end;
end;

end.
